<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly
/**
 * Plugin migrazione  Aruba WP Migration Tool
 *
 * @package     Aruba
 * @author      Aruba
 * @copyright   2022 Aruba
 * @license     GPL-2.0-or-later
 *
 * @wordpress-plugin
 * Plugin Name: Aruba WP Migration Tool
 * Description: This plugin migrates a WordPress installation external to Aruba to an installation internal to Aruba
 * Version:     1.0.2
 * Author:      Aruba
 * License:     GPL v3 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: aruba-wp-migration-tool
 * Domain Path: /languages
 */

require_once "src/environments/environment.php";

require_once "src/migration-plugin/constants/constants.php";
require_once "src/migration-plugin/classes/admin-post-actions.php";
require_once "src/migration-plugin/classes/wp-fs-wrapper.php";
require_once "src/common-classes/migration-progression-info.php";
require_once "src/common-classes/migration-file-manager.php";
require_once "src/migration-plugin/classes/pages/migration-tool-steps/migration-step.php";
require_once "src/migration-plugin/classes/pages/migration-tool-steps/verify-token-step.php";
require_once "src/migration-plugin/classes/pages/migration-tool-steps/check-requirements-step.php";
require_once "src/migration-plugin/classes/pages/migration-tool-steps/check-folders-step.php";
require_once "src/migration-plugin/classes/pages/migration-tool-steps/backup-maintenance-step.php";
require_once "src/migration-plugin/classes/pages/migration-tool-steps/view-migration-progress-step.php";
require_once "src/migration-plugin/classes/pages/page.php";
require_once "src/migration-plugin/classes/pages/migrations-tool-page.php";
require_once "src/migration-plugin/classes/pages/howitworks-page.php";
require_once "src/migration-plugin/custom-handlers/admin-menu-handler.php";
require_once "src/migration-plugin/custom-handlers/admin-post-handler.php";
require_once "src/migration-plugin/custom-handlers/maintenance-handler.php";
//**new aruba migrator cron proccess */
require_once "src/migration-process/classes/aruba-migrator.php";
require_once "src/migration-process/classes/upload-to-orcchestrator.php";
require_once "src/migration-process/classes/migration-log.php";
require_once "src/migration-process/classes/migrator-state/migrator-state.php";
require_once "src/migration-process/classes/migrator-state/migrator-state-enable-maintenance.php";
require_once "src/migration-process/classes/migrator-state/migrator-state-db-dump.php";
require_once "src/migration-process/classes/migrator-state/migrator-state-prepare-packages.php";
require_once "src/migration-process/classes/migrator-state/migrator-state-upload.php";
require_once "src/migration-process/classes/migrator-state/migrator-state-call-orchestrator.php";
require_once "src/migration-process/classes/migrator-state/migrator-state-finalization.php";
require_once "src/migration-process/classes/mysql-db-dumper.php";
require_once "src/migration-process/classes/utilities.php";


function AWMT_sanitize_array( &$array ) {
    foreach ($array as &$value) {
        if( !is_array($value) )	{
            // sanitize if value is not an array
            $value = sanitize_text_field( $value );
        }  else {
            // go inside this function again
            AWMT_sanitize_array($value);
        }
    }
    return $array;
}



clearstatcache();
$GLOBALS["AWMT_MIGRATION_FILE_MANAGER"] = new AWMT_MigrationFileManager(AWMT_FILE_MIGRATION_PATH);
function AWMT_init(){
    ob_start();
    // AVVIO SESSIONE
    if (!headers_sent() && session_id() == "") {
        session_start();
    }
}
// caricamento Js
function AWMT_loadJsScripts($hook)
{
    // è solo un esempio di inclusione js.. in realtà il file non fa nulla se non verificare che jQuery sia presente nel DOM
    wp_enqueue_script('custom_script', plugin_dir_url(__FILE__) . "src/migration-plugin/assets/js/custom.js",[],"1.0",array(
        'strategy'  => 'defer',
        'in_footer'=> true
    ));
    wp_enqueue_style('font-awesome',plugin_dir_url(__FILE__) . 'src/migration-plugin/assets/css/font-awesome.min.css',[],"1.0");
}
// definizione action
add_action('init', 'AWMT_init');

if (file_exists(AWMT_FILE_MIGRATION_PATH) && $GLOBALS["AWMT_MIGRATION_FILE_MANAGER"]->isMaintenanceModeActive()) {
    add_action('init', 'AWMT_handlerMaintenance');
}
// Load the text domain for localization
function AWMT_plugin_load_textdomain() {
    load_plugin_textdomain( 'aruba-wp-migration-tool', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
}

add_action('admin_enqueue_scripts', 'AWMT_loadJsScripts');
add_action( 'plugins_loaded', 'AWMT_plugin_load_textdomain' );                // localization
add_action('admin_menu', 'AWMT_arubaMigratorAdminMenu');                         // menù admin
add_action('admin_post_add_token', 'AWMT_handlerAddToken');                      // handler chiamata validazione tokekn
add_action('admin_post_select_folders', 'AWMT_handlerSelectFolders');            // handler chiamata seleziona cartelle
add_action('admin_post_backup_maintenance', 'AWMT_handlerSetBackupAndMaintenance'); // handler chiamata avvio migrazione
add_action('admin_post_stop_migration', 'AWMT_handlerStopMigration');            // handler chiamata interruzione migrazione
add_action('admin_post_restart_migration', 'AWMT_handlerRestartMigration');      // handler chiamata riavvio migrazione dal 5 step
add_action('admin_post_new_migration', 'AWMT_handlerNewMigration');      // handler chiamata riavvio migrazione dal 1 step

/**
 * Gestione Attivazione Plugin
 */
function AWMT_arubaMigratorPluginActivation(){
    $wpFsWrapper = new AWMT_WPFSWrapper();
    $res = $wpFsWrapper->createFolder(AWMT_PROCESS_FOLDER_PATH);
    if (is_wp_error($res)) {
        return $res;
    }
    // creo il file di log
    $fileLogContent = gmdate("Y/m/d") . "-" . gmdate("h:i:sa") . "\t Attivazione plugin \n";
    $wpFsWrapper->createFile(AWMT_FILE_LOG_PATH, $fileLogContent, 0775);
    // creo il file per la maintenance
    $wpFsWrapper->createFile(AWMT_FILE_MIGRATION_PATH, AWMT_FILE_MIGRATION_DEFAULT_CONTENT, 0775); //VERIFICARE QUESTO STEP IN FASE ATTIVAZIONE

}

register_activation_hook(__FILE__, 'AWMT_arubaMigratorPluginActivation');

/**
 * Gestione Disattivazione Plugin
 */
register_deactivation_hook(__FILE__, 'AWMT_arubaMigratorPluginDeactivation');
function AWMT_arubaMigratorPluginDeactivation(){
    remove_action('init', 'AWMT_init');
    if ($GLOBALS["AWMT_MIGRATION_FILE_MANAGER"]->isMaintenanceModeActive()) {
        remove_action('init', 'AWMT_handlerMaintenance');
    }
    remove_action('admin_post_add_token', 'AWMT_handlerAddToken');
    remove_action('admin_post_select_folders', 'AWMT_handlerSelectFolders');
    remove_action('admin_post_backup_maintenance', 'AWMT_handlerSetBackupAndMaintenance');
    remove_action('admin_post_stop_migration', 'AWMT_handlerStopMigration');
    remove_action('admin_post_restart_migration', 'AWMT_handlerRestartMigration');
    remove_action('new_migration', 'AWMT_handlerNewMigration');
    remove_action('admin_enqueue_scripts', 'AWMT_loadJsScripts');
    remove_action( 'plugins_loaded', 'AWMT_plugin_load_textdomain' );
    remove_action( 'rest_api_init', 'migration_update' );
    $wpFsWrapper = new AWMT_WPFSWrapper();
    $wpFsWrapper->deleteFolder(AWMT_PROCESS_FOLDER_PATH);
}

add_action("wp_ajax_migration_detail", "AWMT_migration_detail");
function AWMT_migration_detail() {
    $detail_api_url=AWMT_API_URL."detail";
    $jwt=$_SESSION['wp2wp-jwt'];

    $headers = array(
        'Content-Type' => 'application/json',
        'Authorization' => "Bearer " . $jwt
    );

    $response = WpOrg\Requests\Requests::get($detail_api_url,$headers);

    $data = $response->decode_body();
    echo wp_json_encode($data);
    die();
}

add_action("wp_ajax_preparemigrationdetail", "AWMT_preparemigrationdetail");
function AWMT_preparemigrationdetail(){
    echo wp_json_encode(['idStatus'=>$GLOBALS["AWMT_MIGRATION_FILE_MANAGER"]->getMigrationFileStatus()]);
    die();
}

add_action("wp_ajax_start_step_process", "AWMT_start_step_process");
function AWMT_start_step_process(){

    $status=AWMT_AdminPostActions::startStepProcess(absint($_REQUEST['step']));// phpcs:ignore WordPress.Security.NonceVerification.Recommended
    echo wp_json_encode(["step"=>sanitize_text_field(absint($_REQUEST['step'])),'status'=>$status]);// phpcs:ignore WordPress.Security.NonceVerification.Recommended
    die();
}

add_action("wp_ajax_setmigrationcomplete", "AWMT_setmigrationcomplete");
function AWMT_setmigrationcomplete(){
    $GLOBALS["AWMT_MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_COMPLETED", "true");
    echo wp_json_encode(['migration_completed'=>$GLOBALS["AWMT_MIGRATION_FILE_MANAGER"]->isMigrationCompleted()]);
    die();
}

add_action("wp_ajax_setmigrationerror", "AWMT_setmigrationerror");
function AWMT_setmigrationerror(){

    $GLOBALS["AWMT_MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_ERROR", "true");
    echo wp_json_encode(['migration_error'=>$GLOBALS["AWMT_MIGRATION_FILE_MANAGER"]->isMigrationError()]);
    die();
}

add_action("wp_ajax_setmantenance", "AWMT_setmantenance");
function AWMT_setmantenance(){
    //check_ajax_referer( 'backup_maintenance', 'security' );
	// phpcs:ignore WordPress.Security.NonceVerification.Recommended
	$GLOBALS["AWMT_MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_PAGE_ACTIVE", esc_attr($_REQUEST['mantenancestatus']));
    echo wp_json_encode(['migration_mantenance'=>$GLOBALS["AWMT_MIGRATION_FILE_MANAGER"]->isMaintenanceModeActive()]);
    die();
}



add_action("wp_ajax_setmigrationexited", "_setmigrationexited");
function _setmigrationexited(){
	$GLOBALS["AWMT_PROCESS_LOG"]->writeLog("Migration Process - ENDED BY EXIT ");
	$GLOBALS["AWMT_MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_EXITED", 'true');
	$_step=$GLOBALS["AWMT_MIGRATION_FILE_MANAGER"]->getMigrationStep();

	AWMT_AdminPostActions::goToStep($_step);
}