<?php
/**
 * Plugin migrazione  Aruba WP Migration Tool
 *
 * @package     Aruba
 * @author      Aruba
 * @copyright   2022 Aruba
 * @license     GPL-2.0-or-later
 *
 * @wordpress-plugin
 * Plugin Name: Aruba WP Migration Tool
 * Description: This plugin migrates a WordPress installation external to Aruba to an installation internal to Aruba
 * Version:     1.0.0
 * Author:      Aruba
 * License:     GPL v2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: aruba-wp-migration-tool
 * Domain Path: /languages
 */

ini_set('max_execution_time', '3000');

$WP_ENV = "Prod";

// Autoloader relative path to this PHP file
//require_once __DIR__ . '/vendor/autoload.php';

if ( $WP_ENV === "development") {
    require_once "src/environments/environment.local.php";
} elseif ( $WP_ENV === "testing") {
    require_once "src/environments/environment.test.php";
}else {
    require_once "src/environments/environment.prod.php";
}

require_once "src/migration-plugin/constants/constants.php";
require_once "src/migration-plugin/classes/admin-post-actions.php";
require_once "src/migration-plugin/classes/wp-fs-wrapper.php";
require_once "src/common-classes/migration-progression-info.php";
require_once "src/common-classes/migration-file-manager.php";
require_once "src/migration-plugin/classes/pages/migration-tool-steps/migration-step.php";
require_once "src/migration-plugin/classes/pages/migration-tool-steps/verify-token-step.php";
require_once "src/migration-plugin/classes/pages/migration-tool-steps/check-requirements-step.php";
require_once "src/migration-plugin/classes/pages/migration-tool-steps/check-folders-step.php";
require_once "src/migration-plugin/classes/pages/migration-tool-steps/backup-maintenance-step.php";
require_once "src/migration-plugin/classes/pages/migration-tool-steps/view-migration-progress-step.php";
require_once "src/migration-plugin/classes/pages/page.php";
require_once "src/migration-plugin/classes/pages/migrations-tool-page.php";
require_once "src/migration-plugin/classes/pages/howitworks-page.php";
require_once "src/migration-plugin/custom-handlers/admin-menu-handler.php";
require_once "src/migration-plugin/custom-handlers/admin-post-handler.php";
require_once "src/migration-plugin/custom-handlers/maintenance-handler.php";
//**new aruba migrator cron proccess */
require_once "src/migration-process/classes/aruba-migrator.php";
require_once "src/migration-process/classes/upload-to-orcchestrator.php";
require_once "src/migration-process/classes/migration-log.php";
require_once "src/migration-process/classes/migrator-state/migrator-state.php";
require_once "src/migration-process/classes/migrator-state/migrator-state-enable-maintenance.php";
require_once "src/migration-process/classes/migrator-state/migrator-state-db-dump.php";
require_once "src/migration-process/classes/migrator-state/migrator-state-prepare-packages.php";
require_once "src/migration-process/classes/migrator-state/migrator-state-upload.php";
require_once "src/migration-process/classes/migrator-state/migrator-state-call-orchestrator.php";
require_once "src/migration-process/classes/migrator-state/migrator-state-finalization.php";
require_once "src/migration-process/classes/mysql-db-dumper.php";
require_once "src/migration-process/classes/utilities.php";

clearstatcache();

$GLOBALS["MIGRATION_FILE_MANAGER"] = new MigrationFileManager(FILE_MIGRATION_PATH);

function init(){
    ob_start();
    // AVVIO SESSIONE
    if (!headers_sent() && session_id() == "") {
        session_start();
    }
}

// caricamento Js
function loadJsScripts($hook)
{
    // è solo un esempio di inclusione js.. in realtà il file non fa nulla se non verificare che jQuery sia presente nel DOM
    wp_enqueue_script('custom_script', plugin_dir_url(__FILE__) . "src/migration-plugin/assets/js/custom.js");
    wp_enqueue_style('font-awesome','https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/css/font-awesome.min.css');
}
// definizione action
add_action('init', 'init');

if (file_exists(FILE_MIGRATION_PATH) && $GLOBALS["MIGRATION_FILE_MANAGER"]->isMaintenanceModeActive()) {
    add_action('init', 'handlerMaintenance');
}
// Load the text domain for localization
function my_plugin_load_textdomain() {
    load_plugin_textdomain( 'aruba-wp-migration-tool', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
}



add_action('admin_enqueue_scripts', 'loadJsScripts');
add_action( 'plugins_loaded', 'my_plugin_load_textdomain' );                // localization
add_action('admin_menu', 'arubaMigratorAdminMenu');                         // menù admin
add_action('admin_post_add_token', 'handlerAddToken');                      // handler chiamata validazione tokekn
add_action('admin_post_select_folders', 'handlerSelectFolders');            // handler chiamata seleziona cartelle
add_action('admin_post_backup_maintenance', 'handlerSetBackupAndMaintenance'); // handler chiamata avvio migrazione
add_action('admin_post_stop_migration', 'handlerStopMigration');            // handler chiamata interruzione migrazione
add_action('admin_post_restart_migration', 'handlerRestartMigration');      // handler chiamata riavvio migrazione dal 5 step
add_action('admin_post_new_migration', 'handlerNewMigration');      // handler chiamata riavvio migrazione dal 1 step

/*add_action( 'rest_api_init', function(){

    register_rest_route( WP2WP_PLUGIN_DIR_NAME . DIRECTORY_SEPARATOR . "v1", CRON_JOB_ENDPOINT, array(
        'methods'  => 'GET',
        'callback' => 'test_function',//'migration_update'
    ) );


} );*/

function test_function()
{
    //$GLOBALS["PROCESS_LOG"]->writeLog("entrato in test function");
    $adminPostActionCrone = new AdminPostActions();
    return $adminPostActionCrone::migration_update();

}

//add_action('migration_update', 'scheduling_cron_job');

/**
 * Gestione Attivazione Plugin
 */
function arubaMigratorPluginActivation()
{
    $wpFsWrapper = new WPFSWrapper();
    $res = $wpFsWrapper->createFolder(PROCESS_FOLDER_PATH);
    if (is_wp_error($res)) {
        return $res;
    }

    // creo il file di log
    $fileLogContent = date("Y/m/d") . "-" . date("h:i:sa") . "\t Attivazione plugin \n";
    $wpFsWrapper->createFile(FILE_LOG_PATH, $fileLogContent, 0775);
    // creo il file per la maintenance
    $wpFsWrapper->createFile(FILE_MIGRATION_PATH, FILE_MIGRATION_DEFAULT_CONTENT, 0775); //VERIFICARE QUESTO STEP IN FASE ATTIVAZIONE
    

}



register_activation_hook(__FILE__, 'arubaMigratorPluginActivation');

/**
 * Gestione Disattivazione Plugin
 */
register_deactivation_hook(__FILE__, 'arubaMigratorPluginDeactivation');
function arubaMigratorPluginDeactivation()
{
    remove_action('init', 'init');
    if ($GLOBALS["MIGRATION_FILE_MANAGER"]->isMaintenanceModeActive()) {
        remove_action('init', 'handlerMaintenance');
    }
    remove_action('admin_post_add_token', 'handlerAddToken');
    remove_action('admin_post_select_folders', 'handlerSelectFolders');
    remove_action('admin_post_backup_maintenance', 'handlerSetBackupAndMaintenance');
    remove_action('admin_post_stop_migration', 'handlerStopMigration');
    remove_action('admin_post_restart_migration', 'handlerRestartMigration');
    remove_action('new_migration', 'handlerNewMigration');
    remove_action('admin_enqueue_scripts', 'loadJsScripts');
    //remove_action('migration_update', 'scheduling_cron_job');
    remove_action( 'plugins_loaded', 'my_plugin_load_textdomain' );
    remove_action( 'rest_api_init', 'migration_update' );
    $wpFsWrapper = new WPFSWrapper();
    $wpFsWrapper->deleteFolder(PROCESS_FOLDER_PATH);

    function my_cron_plugin_deactivate() {
        wp_clear_scheduled_hook( 'migration_update' );
    }
}


add_action("wp_ajax_migration_detail", "_migration_detail");
//add_action("wp_ajax_nopriv_migration_detail", "_migration_detail");

function _migration_detail() {

    $encryptedData = file_get_contents(PROCESS_FOLDER_PATH . DIRECTORY_SEPARATOR . 'sss');
    $processParams = Utilities::decryptArray($encryptedData,"test",md5("test"));
    //var_dump($processParams);
    $detail_api_url=WP2WP_API_URL."detail";
    $jwt=$processParams['jwt'];

    $headers = array(
        'Content-Type' => 'application/json',
        'Authorization' => "Bearer " . $jwt
    );

    $response = WpOrg\Requests\Requests::get($detail_api_url,$headers);

    $data = $response->decode_body();
    //var_dump(json_encode($data));
    echo json_encode($data);
    die();
}



add_action("wp_ajax_preparemigrationdetail", "_preparemigrationdetail");
function _preparemigrationdetail(){
    echo json_encode(['idStatus'=>$GLOBALS["MIGRATION_FILE_MANAGER"]->getMigrationFileStatus()]);
    die();
}

add_action("wp_ajax_start_step_process", "_start_step_process");
function _start_step_process(){
    $status=AdminPostActions::startStepProcess($_REQUEST['step']);
    echo json_encode(["step"=>$_REQUEST['step'],'status'=>$status]);
    die();
}

add_action("wp_ajax_setmigrationcomplete", "_setmigrationcomplete");
function _setmigrationcomplete(){
    $GLOBALS["MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_COMPLETED", "true");
    echo json_encode(['migration_completed'=>$GLOBALS["MIGRATION_FILE_MANAGER"]->isMigrationCompleted()]);
    die();
}

add_action("wp_ajax_setmigrationerror", "_setmigrationerror");
function _setmigrationerror(){

    $GLOBALS["MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_ERROR", "true");
    echo json_encode(['migration_error'=>$GLOBALS["MIGRATION_FILE_MANAGER"]->isMigrationError()]);
    die();
}


add_action("wp_ajax_setmantenance", "_setmantenance");
function _setmantenance(){
    $GLOBALS["MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_PAGE_ACTIVE", $_REQUEST['mantenancestatus']);
    echo json_encode(['migration_mantenance'=>$GLOBALS["MIGRATION_FILE_MANAGER"]->isMaintenanceModeActive()]);
    die();
}