<?php

/**
 * Gestisce la lettura e la scrittura delle proprietà presenti nel file di migrazione .wp2wp-migration
 */
class MigrationFileManager
{
    private $migrationFilePath;
    function __construct($migrationFilePath)
    {
        $this->migrationFilePath = $migrationFilePath;
    }

    public function isMaintenanceModeActive()
    {
        $res = $this->getValue("WP2WP_MIGRATION_PAGE_ACTIVE");
        return $res === "true";
    }

    public function isMigrationActive()
    {
        $state = $this->getMigrationState();
        return $state !== -1;
    }

    public function isMigrationError()
    {
        $res = $this->getValue("WP2WP_MIGRATION_ERROR");
        return $res === "true";
    }

    public function isMigrationKilledByUser()
    {
        $res = $this->getValue("WP2WP_MIGRATION_KILLED_BY_USER");
        return $res === "true";
    }

    public function isMigrationCompleted()
    {
        $res = $this->getValue("WP2WP_MIGRATION_COMPLETED");
        return $res === "true";
    }

    public function getMigrationState()
    {
        $res = $this->getValue("WP2WP_MIGRATION_STATE");
        $res = intval($res);
        return $res;
    }

    public function getMigrationStateName()
    {
        $res = $this->getValue("WP2WP_MIGRATION_CURRENT_STATE_NAME");
        return $res;
    }

    public function getMigrationFileStatus()
    {
        $res = $this->getValue("WP2WP_MIGRATION_FILES_STATUS");
        return $res;
    }

    public function getMigrationProgressInfo()
    {
        $isError = $this->isMigrationError();
        $isKilledByUser = $this->isMigrationKilledByUser();
        $isCompleted = $this->isMigrationCompleted();
        $migrationState = $this->getMigrationState();
        $migrationStateName = $this->getMigrationStateName();
        return new MigrationProgressionInfo($isError, $isCompleted, $isKilledByUser, $migrationState, $migrationStateName);
    }

    public function isMigrationStarted()
    {
        $isError = $this->isMigrationError();
        $isActive = $this->isMigrationActive();
        $isCompleted = $this->isMigrationCompleted();
        return $isError || $isCompleted || $isActive;
    }

    public function getMigrationProcessID()
    {
        $res = $this->getValue("WP2WP_MIGRATION_PROCESS_ID");
        return $res;
    }

    public function updateMigrationKey($k, $v)
    {
        $data = file($this->migrationFilePath);
        $newData = [];
        foreach ($data as $row) {
            $newRow = stristr($row, $k) ? "$k=$v\n" : $row;
            array_push($newData, $newRow);
        }
        file_put_contents($this->migrationFilePath, $newData);
    }

    private function getValue($searchKey)
    {
        $data = file($this->migrationFilePath);
        $res = null;

        foreach ($data as $key => $raw) {
            if (stristr($raw, $searchKey)) {
                $raw = trim(preg_replace('/\s+/', ' ', $raw));
                $r = explode("=", $raw);
                $res = $r[1];
            }
        }
        return $res;
    }
}
