<?php

use ParagonIE\Sodium\Core\Util;

class AdminPostActions
{
    private function init()
    {
        $testx = "";
    }
    /**
     * Gestise i redirect ai vari step della migrazione
     */
    public static function goToStep($step, $getArgs = "")
    {
        if (isset(WP2WP_STEPS_URL[$step])) {
            wp_redirect(admin_url(WP2WP_STEPS_URL[$step]) . $getArgs);
            exit;
        }
    }

    /**
     * Permette di verificare la validità del token di migrazione
     * Questa chiamata viene fatta dal form allo step 1, e permette di andare allo step 2 (se non ci sono errori)
     */
    public static function insertToken()
    {
        if (empty($_REQUEST['token-input'])) {
            self::goToStep(1);
        }

        $params = [
            "token" => $_REQUEST['token-input']
        ];
        //die(var_dump(WP2WP_API_URL . "token/verify/" . $_REQUEST['token-input']));
        $res = wp_remote_post(
            WP2WP_API_URL . "token/verify/" /*. $_REQUEST['token-input']*/,
            array(
                'method' => 'POST',
                'timeout' => 45,
                'headers' => [
                   'Content-Type' => 'application/json',
                    'Accept' => '*/*'
                ],
                'body' => json_encode($params)
            )
        );

        $resp_code = wp_remote_retrieve_response_code($res);
        $resp_body = wp_remote_retrieve_body($res);
        $data = json_decode($resp_body, true);

      // die(var_dump($resp_code,wp_remote_retrieve_body($res)));
//die(var_dump(wp_remote_retrieve_body($res)));
        if ($resp_code!==200) {
            if (isset($data['errorCode'])) {
                self::goToStep(1, "&err-inv-token=" . $data['errorCode']);
            }elseif ($resp_code==400) {
                self::goToStep(1, "&err-inv-token=42001" );
            }else{
                self::goToStep(1, "&err-gen=42400");
            }
        }else {
                $_SESSION['wp2wp-migration-token'] = $_REQUEST['token-input'];
                $_SESSION['wp2wp-jwt'] = $data["access_token"];
                self::goToStep(2);
        }
    }

    /**
     * Vengono collezionate le cartelle ed i file selezionati dall utente allo step 3
     * Questa chiamata viene fatta dal form allo step 3, e permette di andare allo step 4 (se non ci sono errori)
     */
    public static function selectFolders()
    {
        $foldersToCopy = $_SESSION["required-folders"];
        unset($_SESSION["required-folders"]);
        unset($_REQUEST["action"]);
       // $GLOBALS["FOLDER_LOG"] = new MigrationLog(FOLDER_LOG_PATH);

        foreach ($_REQUEST as $key => $val) {

            array_push($foldersToCopy, ["name" => $key, "path" => $val]);
        }

        $_SESSION['wp2wp-folders'] = $foldersToCopy;
        //$GLOBALS["FOLDER_LOG"]->writeLog("-----------------SELECTED FOLDER----------------");
        foreach($foldersToCopy as $key => $val){
          //  $GLOBALS["FOLDER_LOG"]->writeLog("name: ".$key." path: ".$val['path']);
        }
        //$GLOBALS["FOLDER_LOG"]->writeLog("-----------------SELECTED FOLDER----------------");
        self::goToStep(4);
    }

    /**
     * Vengono collezionate le scelte dell utente allo step 4, in merito all'impostazione della maintenance e backup 
     * Qui viene avviato il processo che si occupa dei vari stati di migrazione
     * Questa chiamata viene fatta dal form allo step 4, e permette di andare allo step 5 (se non ci sono errori)
     */
    public static function setBackupAndMaintenance()
    {
        $_SESSION['wp2wp-active-backup'] = isset($_REQUEST["activeBackup"]);
        $_SESSION['wp2wp-active-maintenance'] = isset($_REQUEST["activeMaintenance"]);

        global $wpdb;
        $dbConn = [
            "dbname" => $wpdb->dbname,
            "dbuser" => $wpdb->dbuser,
            "dbpassword" => $wpdb->dbpassword,
            "dbhost" => $wpdb->dbhost
        ];

        if (!self::generateMigrationFolder()) {
            self::goToStep(4, "&err-gen-folder=1");
            return;
        } 

        $processParams = [
            "token" => $_SESSION['wp2wp-migration-token'],
            "apiUrl" => WP2WP_API_URL,
            "jwt" => $_SESSION['wp2wp-jwt'],
            "maintenance" => $_SESSION['wp2wp-active-maintenance'],
            "backup" => $_SESSION['wp2wp-active-backup'],
            "foldersToCopy" => $_SESSION['wp2wp-folders'],
            "foldersToExclude" => [PROCESS_FOLDER_PATH,WP2WP_PLUGIN_DIR_PATH], //
            "migrationFolderPath" => MIGRATION_FOLDER_PATH,
            "logFilePath" => FILE_LOG_PATH,
            "migrationFilePath" => FILE_MIGRATION_PATH,
            "dbConn" => $dbConn,
        ];

        if(isset($_SESSION['plugin_folder_exclude'])){
            $processParams["foldersToExclude"]=array_merge($processParams["foldersToExclude"],$_SESSION['plugin_folder_exclude']);
        }

        $encriptData = Utilities::encryptArray($processParams,"test",md5("test")); 
        file_put_contents(PROCESS_FOLDER_PATH . DIRECTORY_SEPARATOR . 'sss', $encriptData);
        $_SESSION['wp2wp-process-params'] = $processParams;
        wp_clear_scheduled_hook( 'migration_update' );
        $GLOBALS["PROCESS_LOG"] = new MigrationLog($processParams["logFilePath"]);

        //self::startProcess(); //ranzato momentabeamente per test upload


        //sleep(1);
        //self::goToStep(5);
    }


    public static function startStepProcess($step){
       return  self::startProcess($step);
    }



    /**
     * Interrompe la migrazione interrompendo il processo (lato plugin) e fa redirect allo step 5, che mostrerà la scermata opportuna
     */
    public static function stopMigration()
    {
        //$pid = $GLOBALS["MIGRATION_FILE_MANAGER"]->getMigrationProcessID();
        //$res = exec("kill -9 $pid", $out);

        $GLOBALS["MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_KILLED_BY_USER", "true");

        // viene svuotata (distrutta e ricreata) la cartella contenente tutti i file di migrazione
        if (!self::generateMigrationFolder()) {
            //TODO: gestire errore?
        }
        // si ritorna allo step 5
        self::goToStep(5);
    }

    /**
     * Fa ripartire la migrazione da capo, dal primo stato (5 step)
     * Potrebbe esser gestito il recupero degli step conclusi, e ripartire da li
     */
    public static function restartMigration()
    {
        //self::startProcess();
        //sleep(1);
        //self::goToStep(5);
        $wpFsWrapper = new WPFSWrapper();
        $wpFsWrapper->createFile(FILE_MIGRATION_PATH, FILE_MIGRATION_DEFAULT_CONTENT, 0775);
        unset($_SESSION['wp2wp-migration-token']);
        unset($_SESSION['wp2wp-process-jwt']);
        unset($_SESSION['wp2wp-folders']);
        unset($_SESSION['wp2wp-active-maintenance']);
        unset($_SESSION['wp2wp-active-backup']);
        unset($_SESSION['wp2wp-process-params']);
        self::goToStep(1);
    }


    /**
     * Pulisce il file .migration e fa ripartire la migrazione da capo, primo step
     */
    public static function newMigration()
    {
        $wpFsWrapper = new WPFSWrapper();
        $wpFsWrapper->createFile(FILE_MIGRATION_PATH, FILE_MIGRATION_DEFAULT_CONTENT, 0775);
        unset($_SESSION['wp2wp-migration-token']);
        unset($_SESSION['wp2wp-process-jwt']);
        unset($_SESSION['wp2wp-folders']);
        unset($_SESSION['wp2wp-active-maintenance']);
        unset($_SESSION['wp2wp-active-backup']);
        unset($_SESSION['wp2wp-process-params']);
        self::goToStep(1);
    }
    public function  startCron()
    {
        //TODO server level shceduler need need a rest remote call fro API to work  (take a look on line 194) we need to send endpoint to orchestrator and delete @ end migration
        // server-level cron job
        //avvia la schedulazione
        if (!wp_next_scheduled('migration_update')) {
            wp_schedule_event(time(), 'daily', 'migration_update');
            //wp_schedule_single_event(time()+7400, 'migration_update');
        }

        //Debug
        $next_run = wp_next_scheduled( 'migration_update' );
        $testdeljob =  date( 'Y-m-d H:i:s', $next_run );
        //Debug
        //creo l'endpoint
        $endpoint =  parse_url( get_site_url(), PHP_URL_HOST ) . DIRECTORY_SEPARATOR . WP2WP_PLUGIN_DIR_NAME . CRON_JOB_ENDPOINT; //   wp_remote_get( 'https://www.example.com/wp-json/my-cron-plugin/v1/cron-job'

        $test = get_rest_url();

        sleep(3);

    }

    public static function migration_update()
    {
        if(!isset($processParams))
        {
            $encryptedData = file_get_contents(PROCESS_FOLDER_PATH . DIRECTORY_SEPARATOR . 'sss');
            $processParams = Utilities::decryptArray($encryptedData,"test",md5("test"));
        }
        $GLOBALS["PROCESS_LOG"] = new MigrationLog($processParams["logFilePath"]);
        $GLOBALS["PROCESS_LOG"]->writeLog('Chiamata cron da orchestrator');

        //TODO proccess params is lost when the session is closed, a method must be created in the API to recover the jwt (params) in this phase (take a look on line 109)
        
        return self::startProcess();

    }
    //Esposizione endpoint
    function register_update_rest_endpoint() {

    }
    private static function startProcess($_step=null)
    {
        if(!isset($processParams))
        {
            $encryptedData = file_get_contents(PROCESS_FOLDER_PATH . DIRECTORY_SEPARATOR . 'sss');
            $processParams = Utilities::decryptArray($encryptedData,"test",md5("test"));
        }

        //return $processParams["logFilePath"];

        $processParams = $_SESSION['wp2wp-process-params'];
        //$migrationParams = base64_decode($processParams);
        //$migrationParams = unserialize($migrationParams);

        $GLOBALS["PROCESS_LOG"] = new MigrationLog($processParams["logFilePath"]);
        $GLOBALS["MIGRATION_FILE_MANAGER"] = new MigrationFileManager($processParams["migrationFilePath"]);

        $GLOBALS["MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_KILLED_BY_USER", "false");
        $GLOBALS["MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_ERROR", "false");
        //$GLOBALS["MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_PROCESS_ID", $pid);
        $GLOBALS["MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_COMPLETED", "false");

        //$GLOBALS["PROCESS_LOG"]->writeLog("Migration Process - Initialization: " . serialize($processParams));

        $migrator = new ArubaMigrator($processParams);

        if(!is_null($_step)){
                return $migrator->start($_step);
        }else{
            $migrator->start();
        }



        //exec call removed due to incompatibily with most of hosting providers security policy
        /*
        if (empty($_SESSION['wp2wp-process-params'])) {
            self::goToStep(1);
        }

        $processParams = $_SESSION['wp2wp-process-params'];

        $processPath = WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . WP2WP_PLUGIN_DIR_NAME . DIRECTORY_SEPARATOR . "src" . DIRECTORY_SEPARATOR . "migration-process" . DIRECTORY_SEPARATOR . "migration-process.php";

        $seralized = serialize($processParams);

        $command =  "php $processPath -m " . base64_encode($seralized) . " > /dev/null 2>&1 & echo $!; ";

        $pid = exec($command, $output);

        $GLOBALS["MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_KILLED_BY_USER", "false");
        $GLOBALS["MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_ERROR", "false");
        $GLOBALS["MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_PROCESS_ID", $pid);
        $GLOBALS["MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_COMPLETED", "false");
        //$_SESSION['wp2wp-active-maintenance']

        */

        sleep(1);
    }

    private static function generateMigrationFolder()
    {
        $wpFsWrapper = new WPFSWrapper();
        $wpFsWrapper->deleteFolder(MIGRATION_FOLDER_PATH);
        $res = $wpFsWrapper->createFolder(MIGRATION_FOLDER_PATH);
        return !is_wp_error($res);
    }

    //NEW FEATURE this method is not longer used s3 upload will be done by public orchestrator
    private static function getS3Credential()
    {
        $params = ["token" => $_SESSION['wp2wp-migration-token']];
        $params = http_build_query($params);
        $res = wp_remote_get(
            WP2WP_API_URL . "migration/storage?$params",
            array('headers' => [
                'Content-Type' => 'application/json',
                'Accept' => "*/*",
                'Authorization' => "Bearer " . $_SESSION["wp2wp-jwt"]
            ])
        );

        $resp_code = wp_remote_retrieve_response_code($res);

        if ($resp_code < 200 || $resp_code > 299) {
            return false;
        }

        if (is_wp_error($res)) {
            $error_message = $res->get_error_message();
            $GLOBALS["PROCESS_LOG"]->writeLog("Migration Process - API Error: $error_message");
            return false;
        }

        $body = wp_remote_retrieve_body($res);

        $s3Conn = json_decode($body);

        return $s3Conn;
    }
}
