<?php

class CheckRequirementsStep extends MigrationStep
{
    function __construct()
    {
        parent::__construct(2, __("Check requirements",'aruba-wp-migration-tool'));
    }

    public function canBeDone()
    {
        return !empty($_SESSION['wp2wp-migration-token']) && !empty($_SESSION['wp2wp-jwt']);
    }

    public function draw()
    {
        // recupero le versioni da sparare all'orchestratore
        global $wp_version;
        global $wpdb;

        // $wp_version || $wpdb->db_version()
        $result = $wpdb->get_results("SELECT VERSION() as version");
        $sqlVersion = $result[0]->version;

        /*$params = [
            //["name" => "wp", "content" => $wp_version],
            ["name" => "wp", "version" => '6.1'], //hack
            ["name" => "php", "version" => phpversion()],
            ["name" => "sql", "version" => $sqlVersion],
            ["name" => "plugins", "content" => $this->getPluginList()]
        ];*/

        $params = array(
            //["name" => "wp", "content" => $wp_version],
            "versions" => $this->getVersions(), //hack
            "plugins" => $this->getPluginList()
        );

        $checkVerified = $this->verifyIntegrityCheck($this->getVersions(), $this->getPluginList());

        if (!$checkVerified) {
            echo "<p>".__("Unable to verify migration requirements","aruba-wp-migration-tool")."</p>";
            return;
        }

        //$requirementsAreValid = $checkVerified["success"] === true;
        //$checks = $checkVerified["checks"];
        $requirementsAreValid = true; //would be better if to come from api
        $pluginsrequirementsAreValid=true;
        echo "<div class='wp2wp-panel wp2wp-mb wp2wp-pl wp2wp-pr'>";
        $_versions = $checkVerified["versions"];
        $_plugins = $checkVerified["plugins"];
        $_FWarning=false;
        //Versions iterator
        foreach ($_versions as $key=>$r) {
            $_class = $r["result"];
            $_message =( ($r["message"]==="OK")?__("Verified",'aruba-wp-migration-tool'):__($r["message"],'aruba-wp-migration-tool'));

            //$title = $r["name"];
            switch($r["name"]){
                case "wp":
                    $title=__('Check WordPress version','aruba-wp-migration-tool');
                    break;
                case "php":
                    $title=__('Check PHP version','aruba-wp-migration-tool');
                    break;
                case "mysql":
                    $title=__('Check Database version','aruba-wp-migration-tool');
                    break;

            }
            if($_class == "Warning"){
                $_FWarning=true;
            }

            if(isset($_class) && ($_class != "Success" && $_class != "Warning")  )
            {
                $requirementsAreValid = false;
            }

            echo "<div class='wp2wp-requirements-list-item wp2wp-pt wp2wp-pb $_class'>
                    <div class='check-title wp2wp-text-bold'>$title</div>
                    <div class='check-message'>$_message</div>
                </div>";
        }
        //plugins iteratot
        $error_Str="";
        foreach ($_plugins as $key=>$r) {
            $_class = $r["result"];

           // $_message = $r["message"];
           // $title = $r["name"];

            if(isset($_class) && $_class != "Success"  ) {
                $pluginsrequirementsAreValid = false;
                //var_dump($r);
                $_class = $r["result"];
                $error_Str.="<div>".__("One or more plugins are not updated to the latest available version. Upgrade to
proceed with the migration.","aruba-wp-migration-tool")."</div>";
            }

        }
        if(!$pluginsrequirementsAreValid){
            echo "<div class='wp2wp-requirements-list-item wp2wp-pt wp2wp-pb Failed'>
                    <div class='check-title wp2wp-text-bold'>" .__('Check plugins installed','aruba-wp-migration-tool'). "</div>
                    <div class='check-message'>$error_Str</div>
                </div>";
        }else{
            echo "<div class='wp2wp-requirements-list-item wp2wp-pt wp2wp-pb $_class'>
                    <div class='check-title wp2wp-text-bold'>" .__('Check plugins installed','aruba-wp-migration-tool'). "</div>
                    <div class='check-message'>" .__('Verified','aruba-wp-migration-tool'). "</div>
                </div>";
        }

        echo "</div>";

        if ($requirementsAreValid && $pluginsrequirementsAreValid){
            if(!$_FWarning){
              echo "<p class='wp2wp-text-success'>" . __("All requirements have been met. Click on continue to proceed with migration.", "aruba-wp-migration-tool") . "</p>";
            }
        }else{
            echo "<p class='wp2wp-text-failed'>" . __("The requirements were not met", "aruba-wp-migration-tool") . "</p>";
        }

        echo "<hr class='wp2wp-mb wp2wp-mt'>";

        // TEMP MOD FOR TESTING
        if (!$requirementsAreValid || !$pluginsrequirementsAreValid) {
            echo "<a class='button-primary pull-right' disabled href='#'>" . __("Continue", "aruba-wp-migration-tool") . "</a>";
        }else {
            echo "<a class='button-primary pull-right' id='submitbutton' href='?page=aruba-wp-migration-tool-migration&step=3'>" . __("Continue", "aruba-wp-migration-tool") . "&nbsp;<i class='fa fa-circle-o-notch fa-spin' style='display:none'></i></a>";
            echo '
                <script>
                    jQuery(document).ready(function(){
                        
                        jQuery("#submitbutton").on("click",function(e,o){
                            jQuery("i",o).show(); 
                        });
                        
                        
                    })

                </script>';
        }
        // TEMP MOD FOR TESTING
        echo "<a class='button-secondary pull-right wp2wp-mr' href='?page=aruba-wp-migration-tool-migration&step=1'>".__("Back","aruba-wp-migration-tool")."</a>";
        //TEMP MOD PER TESTING
        $_SESSION["wp2wp-valid-requirements"] = ($requirementsAreValid && $pluginsrequirementsAreValid)?true:false;
        //$_SESSION["wp2wp-valid-requirements"]=true;
    }

    /**
     * return installed plugin name, version, maxversion as list of jeson data 
     */
    private function getPluginList()
    {
        // da sistemare

        $all_plugins = get_plugins();
        // Get active plugins
        $active_plugins = get_option('active_plugins');

        $update_plugins = get_site_transient( 'update_plugins' );
        $list = [];
        $plugin_folder_exclude=[];
        foreach($all_plugins as  $file=>$value ) {

            if (array_search($file, $active_plugins, true) !== false) {
                if (isset($update_plugins->response[$file])) {
                    $plugin = array(
                        "latestVersion" => $update_plugins->response[$file]->new_version,
                        "version" => $update_plugins->checked[$update_plugins->response[$file]->plugin],
                        "name" => $value['Name']
                    );
                } else {
                    $plugin = array(
                        "latestVersion" => $value['Version'],
                        "version" => $value['Version'],
                        "name" => $value['Name']
                    );
                }
                array_push($list, $plugin);
            }else{

                array_push($plugin_folder_exclude, str_replace('\\', '/', str_replace(site_url('/'), ABSPATH, plugins_url()).DIRECTORY_SEPARATOR.plugin_dir_path($file)) );

            }
        }

        $_SESSION['plugin_folder_exclude']=$plugin_folder_exclude;
       // var_dump($_SESSION['plugin_folder_exclude']);
        return $list;
    }

    private function getVersions()
    {

        global $wpdb;
        $result = $wpdb->get_results("SELECT VERSION() as version");
        $sqlVersion = $result[0]->version;
        // da sistemare
        global $wp_version;
        $versions = array(
            array(
                "name" => "wp",
                "version" => $wp_version
            ),
            array(
                "name" => "php",
                "version" => phpversion()
            ),
            array(
                "name" => "mysql",
                "version" => $sqlVersion
            )
        );

        return $versions;
    }

    /**
     * Chiama Orchestrator per la verifica dei check di inrtegrità
     */
    private function verifyIntegrityCheck($versions, $plugins)
    {
        $body = [
            "token" => $_SESSION['wp2wp-migration-token'],
            "versions" => $versions,
            "plugins" => $plugins
        ];
        $res = wp_remote_post(
            WP2WP_API_URL . "integritycheck/verify",
            array(
                'method' => 'POST',
                'timeout' => 45,
                'headers' => [
                    'Content-Type' => 'application/json',
                    'Accept' => "*/*",
                    'Authorization' => "Bearer " . $_SESSION["wp2wp-jwt"]
                ],
                'body' => json_encode($body)
            )
        );

        $resp_code = wp_remote_retrieve_response_code($res);

        if (is_wp_error($res) ||  $resp_code < 200 || $resp_code > 299) {
            error_log("----- ERROR " . serialize($res));
            return false;
        }

        $resp_body = wp_remote_retrieve_body($res);
        $data = json_decode($resp_body, true);

        return $data;
    }

    /**
     * Ritorna la stringa html relativa la rappresentazione del titolo check in base al tipp
     */
    private function parseCheckTitle($name, $type)
    {
        $message = "";
        if ($type === "version") {
            $message = __("Version control ", "aruba-wp-migration-tool") . $name;
        }
        if ($type === "denylist") {
            $message = __("Check installed plugins", "aruba-wp-migration-tool");
        }
        return $message;
    }

    /**
     * Ritorna la stringa html relativa la rappresentazione del check in base al tipp
     */
    private function parseCheckDetails($check, $sended)
    {
        $message = "";
        $type = $check["type"];
        $details = $check["details"];
        if ($type === "version") {
            $names = array_column($sended, 'name');
            $found_key = array_search($check["name"], $names);
            $version = $sended[$found_key]["content"];
            $message = "[$version]" . __("The version must be between ", "aruba-wp-migration-tool") . $details["min"] . __(" and ", "aruba-wp-migration-tool") . $details["max"];
        }
        if ($type === "denylist") {
            if (is_array($details)) {
                $notAllowed = $details["notAllowed"];
                $message = __("The following plugins are not supported: " , "aruba-wp-migration-tool");
                foreach ($notAllowed as $plugin) {
                    $message = $message . "<br> - " . $plugin;
                }
            }
            if (is_string($details)) {
                $message = $details;
            }
        }
        return $message;
    }
}
