<?php

class ViewMigrationProgressStep extends MigrationStep
{
    private $reloadTime;
    function __construct()
    {
        parent::__construct(5, __("Migrate",'aruba-wp-migration-tool')  );
        $this->reloadTime = 5000;
    }
    public function canBeDone()
    {
        return $GLOBALS["MIGRATION_FILE_MANAGER"]->isMigrationStarted();
    }

    public function draw()
    {

        $migrationInfo = $GLOBALS["MIGRATION_FILE_MANAGER"]->getMigrationProgressInfo();

        $message = $migrationInfo->migrationStateName;

        if ($migrationInfo->isKilledByUser === true) {
            $this->killedByUserPanel($message);
            return;
        }
        if ($migrationInfo->isError === true) {
            $this->errorPanel($message);
            return;
        }
        if ($migrationInfo->isCompleted === true) {
            $this->migrationCompletedPanel();
            return;
        }

        // per il calcolo dell'avanzamento della progress bar
        $progress = ($migrationInfo->migrationState + 1) * 10;
        $this->migrationProgressPanel($message, $progress);
    }

    private function killedByUserPanel($message)
    {
        $revoke_api_url=WP2WP_API_URL."revoke";

        $headers = array(
            'Content-Type' => 'application/json',
            'Authorization' => "Bearer " . $_SESSION["wp2wp-jwt"],
            'token' => $_SESSION['wp2wp-migration-token']
        );

        $response = WpOrg\Requests\Requests::delete($revoke_api_url,$headers);

        $data = $response->decode_body();


        echo "<div class='wp2wp-panel wp2wp-text-center' style='padding: 20px; max-width:590px;'>
                <span class='wp2wp-icon error'></span>
                <h2 class='wp2wp-text-center' style='font-size: 22px;'>" . __("Migration aborted", "aruba-wp-migration-tool") . "</h2>
                 <p class='wp2wp-text-bold'>" . __("The procedure was stopped by the user during the", "aruba-wp-migration-tool") . "&nbsp;".__($message,'aruba-wp-migration-tool')."</p>
                <hr class='wp2wp-mb wp2wp-mt' style='margin-left: 18%;margin-right: 18%;' >
                 
                <p style='margin-left: 18%;margin-right: 18%;'>
                <strong>".__("I want to keep the mode maintenance", "aruba-wp-migration-tool")."</strong>&nbsp;&nbsp;
                <span >
                <label class='switch maintenance'>
                  <input type='checkbox' name='maintenance' id='status-maintenance' ".  ( ($GLOBALS["MIGRATION_FILE_MANAGER"]->isMaintenanceModeActive())?"checked":"" )." >
                  <span class='slider round'></span>
                  </label>
                  </span>
                </p>
                <p style='margin-left: 18%;margin-right: 18%;text-align: justify;'>
                " . __("We recommend leaving your site in maintenance until you finish publishing on Aruba.", "aruba-wp-migration-tool") . "
                 </p>
                <p style='margin-left: 18%;margin-right: 18%; text-align: justify;'>
                " . __("By activating the maintenance mode, this site will be visible and any operation carried out will not be migrated to the site hosted by Aruba.", "aruba-wp-migration-tool") . "
                </p>
                
                <form method='GET' action='" . esc_url(admin_url('admin-post.php')) . "'>
                    <input type='hidden' name='action' value='restart_migration'>
                    <input class='button-secondary' type='submit' type='submit' value='" . __("Start the migration process again", "aruba-wp-migration-tool") . "'>
                </form>
        </div>";
    }

    private function errorPanel($message)
    {
        echo "<div class='wp2wp-panel wp2wp-text-center' style='padding: 20px; max-width:590px;'>
                <span class='wp2wp-icon error'></span>
                <h2 class='wp2wp-text-center' style='font-size: 22px;'>" . __("Migration failed", "aruba-wp-migration-tool") . "</h2>
                 <p class='wp2wp-text-bold'>" . __("An error occurred during the phase", "aruba-wp-migration-tool") . "&nbsp;".__($message,'aruba-wp-migration-tool')."</p>
                <hr class='wp2wp-mb wp2wp-mt' style='margin-left: 18%;margin-right: 18%;' >
                 <p>
                    " .  __("We suggest you carry out the procedure again or", "aruba-wp-migration-tool") ."<br>
                    <a href='" . ARUBA_HELP_URL . "' target='_blank'>" . __("Please see our dedicated Aruba Support page", "aruba-wp-migration-tool") . "</a>
                </p>
                <p style='margin-left: 18%;margin-right: 18%;'>
                <strong>".__("I want to keep the mode maintenance", "aruba-wp-migration-tool")."</strong>&nbsp;&nbsp;
                <span >
                <label class='switch maintenance'>
                  <input type='checkbox' name='maintenance' id='status-maintenance' ".  ( ($GLOBALS["MIGRATION_FILE_MANAGER"]->isMaintenanceModeActive())?"checked":"" )." >
                  <span class='slider round'></span>
                  </label>
                  </span>
                </p>
                <p style='margin-left: 18%;margin-right: 18%;text-align: justify;'>
                " . __("We recommend leaving your site in maintenance until you finish publishing on Aruba.", "aruba-wp-migration-tool") . "
                 </p>
                <p style='margin-left: 18%;margin-right: 18%; text-align: justify;'>
                " . __("By deactivating the maintenance mode, this site will be visible and any operation carried out will not be migrated to the site hosted by Aruba.", "aruba-wp-migration-tool") . "
                </p>
                
                <form method='GET' action='" . esc_url(admin_url('admin-post.php')) . "'>
                    <input type='hidden' name='action' value='new_migration'>
                    <input class='button-secondary' type='submit' type='submit' value='" . __("Start the migration process again", "aruba-wp-migration-tool") . "'>
                </form>
        </div>";

    }

    private function migrationCompletedPanel()
    {
        echo "<div class='wp2wp-panel wp2wp-text-center' style='padding: 20px; max-width:590px;'>
                <span class='wp2wp-icon ok'></span>
                <h2 class='wp2wp-text-center' style='font-size: 22px;'>" . __("Migration complete", "aruba-wp-migration-tool") . "</h2>
                <hr class='wp2wp-mb wp2wp-mt' style='margin-left: 18%;margin-right: 18%;' >
                <p style='margin-left: 18%;margin-right: 18%;'>
                <strong>".__("I want to keep the mode maintenance", "aruba-wp-migration-tool")."</strong>&nbsp;&nbsp;
                <span >
                <label class='switch maintenance'>
                  <input type='checkbox' name='maintenance' id='status-maintenance' ".  ( ($GLOBALS["MIGRATION_FILE_MANAGER"]->isMaintenanceModeActive())?"checked":"" )." >
                  <span class='slider round'></span>
                  </label>
                  </span>
                </p>
                <p style='margin-left: 18%;margin-right: 18%;text-align: justify;'>
                " . __("We recommend leaving your site in maintenance until you finish publishing on Aruba.", "aruba-wp-migration-tool") . "
                 </p>
                <p style='margin-left: 18%;margin-right: 18%; text-align: justify;'>
                " . __("By deactivating the maintenance mode, this site will be visible and any operation carried out will not be migrated to the site hosted by Aruba.", "aruba-wp-migration-tool") . "
                </p>
                <a class='button' style='margin-top:24px;margin-bottom: 16px;' href='".ARUBA_CLIENT_AREA_URL."' target='_blank'>".__("Log in to the Aruba customer area","aruba-wp-migration-tool"). "</a>
                <form method='GET' action='" . esc_url(admin_url('admin-post.php')) . "' style='margin-bottom: 30px;'>
                    <input type='hidden' name='action' value='new_migration'>
                    <input class='button-secondary' type='submit' type='submit' value='" . __("Start the migration process again", "aruba-wp-migration-tool") . "'>
                </form>
        </div>";
    }

    private function migrationProgressPanel($message, $progress)
    {

        echo "<script>

jQuery(document).ready(function(){
      var pollInterval=false;
      var s=false;
      var ps=false;
      var c_ss='';
      var status_string='';
      var c_index=1;

        var poll = function() {
          jQuery.ajax({
            url: '".admin_url('admin-ajax.php?action=migration_detail')."',
            dataType: 'json',
            type: 'get',
            success: function(data) { // check if available
              //console.log(data);
              s=parseInt(data.idStatus);
              ps=parseInt(data.idProgressStatus);
              //console.log(s,ps);
               switch(s){
                 case 2 :
                 case 3 :
                   translate_status(s,ps);break;
                 case 4 :
                 case 5 :
                   translate_error(s); break;
                 default: /*console.log('Error!');
                   jQuery('#wp2wp.wrap>.notice').after('<div class=\"notice notice-error\" >'+
           '<p>".esc_html(__("An application error has occurred. We suggest you try again in a few minutes. For more information",'aruba-wp-migration-tool'))."<a href=\'#\'>".__("contact technical support",'aruba-wp-migration-tool')." </a></p></div>');
            
                          clearInterval(pollInterval); 
                          jQuery('.actiontitle span').hide();*/
                          clearInterval(pollInterval);
               jQuery('.actiontitle span').hide();
                jQuery.ajax({
                        type: 'GET',
                        url: '".esc_url(admin_url('admin-ajax.php?action=setmigrationerror'))."',
                        async : false,
                        success:function (dataprocess) {
                                window.location.href='".admin_url(WP2WP_STEPS_URL[5])."';
                        }
                  });
                          break;
                 
               }
            },
            error: function() { // error logging
             
              clearInterval(pollInterval);
               jQuery('.actiontitle span').hide();
                jQuery.ajax({
                        type: 'GET',
                        url: '".esc_url(admin_url('admin-ajax.php?action=setmigrationerror'))."',
                        async : false,
                        success:function (dataprocess) {
                                window.location.href='".admin_url(WP2WP_STEPS_URL[5])."';
                        }
                  });
            }
          });
        };
      
        var translate_status=function(s,process_status_id){
         
          if(s!==3 &&( process_status_id>=200 && process_status_id <210)){
            c_index=0;
          }else if(s!==3 &&( process_status_id>=210 && process_status_id <220)){
              c_index=1;
          }else if (s!==3 &&( process_status_id>=220 && process_status_id <=225)){
              c_index=2;
          }else if (s!==3 &&( process_status_id>225  && process_status_id <235)){
              c_index=3;
          }else if (s!==3 &&( process_status_id>=235 && process_status_id <=290)){
              c_index=4;
          }else{
              c_index=5;
             clearInterval(pollInterval);
             jQuery.ajax({
                        type: 'GET',
                        url: '".esc_url(admin_url('admin-ajax.php?action=setmigrationcomplete'))."',
                        async : false,
                        success:function (dataprocess) {
                                window.location.href='".admin_url(WP2WP_STEPS_URL[5])."';
                        }
                  });
          }
          //console.log('index:'+c_index+'->'+c_ss+'!=='+status_string+'='+(c_ss!==status_string));
          if(c_ss!==c_index ){
              cur_obj_index= jQuery('.status-dot.orange').attr('id');
               //console.log('cur_obj_index:'+cur_obj_index);
              for(cur_obj_index; cur_obj_index<=c_index;cur_obj_index++){
                 // console.log('cur_obj_index:'+cur_obj_index+'<='+c_index);
                  if(jQuery('.status-dot#'+cur_obj_index).hasClass('orange'))
                    jQuery('.status-dot#'+cur_obj_index).removeClass('orange');
                  if(jQuery('.status-dot#'+cur_obj_index).hasClass('white'))
                    jQuery('.status-dot#'+cur_obj_index).removeClass('white');
                  if(!jQuery('.status-dot#'+cur_obj_index).hasClass('black'))
                    jQuery('.status-dot#'+cur_obj_index).addClass('black');
                  if(jQuery('.status-dot#'+cur_obj_index+'>.status-label').hasClass('current'))
                    jQuery('.status-dot#'+cur_obj_index+'>.status-label').removeClass('current');
              }
              
              jQuery('.status-dot#'+(c_index)).removeClass('white');
              jQuery('.status-dot#'+(c_index)).addClass('orange');
              jQuery('.status-dot#'+(c_index)+'>.status-label').addClass('current');
              
           c_ss=c_index;
           //c_index++;
          }
        }
        
        var translate_error=function(s){
            clearInterval(pollInterval);
         if(s==4){
           jQuery.ajax({
                        type: 'GET',
                        url: '".esc_url(admin_url('admin-ajax.php?action=setmigrationerror'))."',
                        async : false,
                        success:function (dataprocess) {
                                window.location.href='".admin_url(WP2WP_STEPS_URL[5])."';
                        }
                  });
         }
         if(s==5){
          //console.log('REVOKED');
           jQuery.ajax({
                        type: 'GET',
                        url: '".esc_url(admin_url('admin-ajax.php?action=setmigrationerror'))."',
                        async : false,
                        success:function (dataprocess) {
                                window.location.href='".admin_url(WP2WP_STEPS_URL[5])."';
                        }
                  });
         }
         
        }
        
        pollInterval = setInterval(function() { 
          poll();
          }, 5000);
        poll(); 
        
        
    });
</script>";

        echo "<div class='wp2wp-panel' style='padding: 20px'>
                <h2 class='wp2wp-text-center actiontitle'>".__("Migration in progress","aruba-wp-migration-tool")."
                 <span>.</span><span>.</span><span>.</span>
                </h2>
             
                <div id='LiteralBarraStato'>
                  <div class='status-bar'>
                  
                    <div class='status-line'></div>
                    <div id='0' class='status-dot orange' style='top:0%;'>
                      <div class='status-label current'>".__("Start of migration procedure","aruba-wp-migration-tool")." </div> </div>
                   
                    
                     <div class='status-line'></div>
                     <div id='1' class='status-dot white' style='top:20%;'>
                       <div class='status-label'>".__("Content downloading","aruba-wp-migration-tool")." </div> 
                     </div>
            
                    
                     <div class='status-line'></div>
                     <div id='2' class='status-dot white' style='top:40%;'>
                       <div class='status-label'>".__("Antivirus scan","aruba-wp-migration-tool")." </div> 
                     </div>
            
                    
                    <div class='status-line'></div>
                    <div id='3' class='status-dot white' style='top:60%;'>
                      <div class='status-label'>".__("Content transfer","aruba-wp-migration-tool")." </div> 
                    </div>
              
                    
                    <div class='status-line'></div>
                    <div id='4' class='status-dot white' style='top:80%;'>
                      <div class='status-label'>".__("Import, Configuration and Verification","aruba-wp-migration-tool")." </div> 
                    </div>
                
                    
                    <div class='status-line'></div>
                    <div id='5' class='status-dot white' style='top:100%;'>
                      <div class='status-label'>".__("Completed","aruba-wp-migration-tool")." </div> 
                    </div>        
                  </div>
                </div>
                
                <hr class='wp2wp-mt wp2wp-mb'>
                <p class='wp2wp-text-center'>
                   " . __("Based on the size of the folders to be migrated and the connection,","aruba-wp-migration-tool")." 
                    <b>" . __("the procedure may take up to a few hours.", "aruba-wp-migration-tool") . "</b><br>
                    " . __("Leave this page open to monitor the migration status, otherwise you can close it without compromising the completion of the operation.", "aruba-wp-migration-tool") . "
                </p>
                <p class='wp2wp-text-center wp2wp-mb'>" . __("You can continue to follow the progress of the process directly from the Aruba Hosting control panel of the space you are migrating to.", "aruba-wp-migration-tool") . "</p>
                <form method='GET' action='" . esc_url(admin_url('admin-post.php')) . "' style='text-align: right;'>
                    <input type='hidden' name='action' value='stop_migration'>
                    <a class='button-secondary wp2wp-mr' href='" . ARUBA_CLIENT_AREA_URL . "' target='_blank'>" . __("Log in to the Aruba customer area", "aruba-wp-migration-tool") . "</a>
                    <input class='button-secondary' type='submit' type='submit' value='" . __("Stop Migration", "aruba-wp-migration-tool") . "'>
                </form>
            </div>";
    }
}
