<?php

class ArubaMigrator
{
    private $token;
    private $apiUrl;
    private $jwt;
    private $migrationFolderPath;
    private $enableMaintenance;
    private $shouldBackup;
    private $foldersToCopy;
    private $foldersToExclude;
    private $dbConn;
    //private $s3Conn;

    function __construct($migrationParams)
    {
        $this->token = $migrationParams["token"];
        $this->apiUrl = $migrationParams["apiUrl"];
        $this->jwt = $migrationParams["jwt"];
        $this->migrationFolderPath = $migrationParams["migrationFolderPath"];
        $this->enableMaintenance = $migrationParams["maintenance"];
        $this->shouldBackup = $migrationParams["backup"];
        $this->foldersToCopy = $migrationParams["foldersToCopy"];
        $this->foldersToExclude = isset($migrationParams["foldersToExclude"]) ? $migrationParams["foldersToExclude"] : [];
        $this->dbConn = $migrationParams["dbConn"];
        //$this->s3Conn = $migrationParams["s3Conn"];
    }

    private function execStepsRecursive($steps, $idx)
    {
        if (count($steps) <= $idx) {
            return true;
        }

        $step = $steps[$idx];
        $GLOBALS["MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_STATE", "$idx");

        if (!$step->executeStep()) {
            return false;
        }
        $idx = $idx + 1;
        return $this->execStepsRecursive($steps, $idx);
    }

    public function start($_step=null)
    {
        if(isset( $GLOBALS["PROCESS_LOG"]))
            $GLOBALS["PROCESS_LOG"]->writeLog("Migration Process - START");

        $states = [
            new MigratorStateEnableMaintenance($this->enableMaintenance),
            new MigratorStateDBDump($this->dbConn, $this->migrationFolderPath),
            new MigratorStatePreparePackages($this->foldersToCopy, $this->foldersToExclude, $this->migrationFolderPath),
            new MigratorStateUpload($this->migrationFolderPath, $this->jwt),
            new MigratorStateCallOrchestrator($this->apiUrl, $this->jwt, $this->token, $this->shouldBackup),
            new MigratorStateFinalization($this->migrationFolderPath)
        ];
        if(!is_null($_step)){
            $step = $states[$_step];
            /*if (!$step->executeStep()) {
                $valid= false;
            }*/
            $valid=$step->executeStep();
            $GLOBALS["MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_STATE", "$_step");
           //$GLOBALS["MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_FILES_STATUS", $_step);
            if (!$valid) {
                $GLOBALS["PROCESS_LOG"]->writeLog("Migration Process $_step - ENDED WITH ERROR");
                return false;
            }
        }else{
          $valid = $this->execStepsRecursive($states, 0);
            if (!$valid) {
                $GLOBALS["PROCESS_LOG"]->writeLog("Migration Process - ENDED WITH ERROR");
            }

            $GLOBALS["MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_COMPLETED", "true");
        }

        $GLOBALS["PROCESS_LOG"]->writeLog("Migration Process - END");
    }
}
