<?php

use \Aws\S3\S3Client;
use \Aws\S3\MultipartUploader;
use \Aws\Exception\MultipartUploadException;

class S3ClientWrapper
{

    private $s3;
    private $endpoint;
    private $bucket;
    private $key;
    private $secret;
    function __construct($key, $secret, $endpoint, $bucket)
    {
        $this->endpoint = $endpoint;
        $this->bucket = $bucket;
        $this->key = $key;
        $this->secret = $secret;
    }

    public function init()
    {
        // Instantiate an Amazon S3 client.
        $this->s3 = new S3Client([
            'version' => 'latest',
            'region'  => 'us-east-1',
            'endpoint' => $this->endpoint,
            'use_path_style_endpoint' => false,
            'credentials' => [
                'key'    => $this->key,
                'secret' => $this->secret,
            ],
        ]);
    }

    public function putObject($filename, $filePath)
    {
        $res = true;
        $GLOBALS["PROCESS_LOG"]->writeLog("Migration Process - TRY: [" . $filename . "] ");
        try {
            $this->multipartUpload($filename, $filePath);
        } catch (Aws\S3\Exception\S3Exception $e) {
            echo "There was an error uploading the file.\n";
            $GLOBALS["PROCESS_LOG"]->writeLog("Migration Process - ERROR: [" . $e . "] ");
            $res = false;
        }
        return $res;
    }

    public function multipartUpload($filename, $filePath)
    {
        //Using stream instead of file path
        $source = fopen($filePath, 'rb');
        $uploader = new MultipartUploader($this->s3, $source, [
            'bucket' => $this->bucket,
            'key' => $filename,
        ]);

        do {
            try {
                $result = $uploader->upload();
            } catch (MultipartUploadException $e) {
                rewind($source);
                $uploader = new MultipartUploader($this->s3, $source, [
                    'state' => $e->getState(),
                ]);
            }
        } while (!isset($result));
    }
    public function putObjectOld($filename, $filePath)
    {
        $res = true;
        try {
            $this->s3->putObject([
                'Bucket' => $this->bucket,
                'Key'    => $filename,
                'Body'   => fopen($filePath, 'r'),
                'ACL'    => 'public-read',
            ]);
        } catch (Aws\S3\Exception\S3Exception $e) {
            echo "There was an error uploading the file.\n";
            $GLOBALS["PROCESS_LOG"]->writeLog("Migration Process - ERROR: [" . $e . "] ");
            $res = false;
        }
        return $res;
    }
}
