<?php

class Utilities
{

    static function scanDir($folder)
    {
        $files = scandir($folder);
        if (!$files) {
            return false;
        }
        return array_diff($files, array('.', '..'));
    }

    static function zipFile($fileZipName, $fileToZip, $destinationFolder)
    {
        return self::zipFiles($fileZipName, [$fileToZip], $destinationFolder);
    }
    static function zipAddFile($file, $fileToAdd)
    {   
        $zip = new ZipArchive();
        if ($zip->open($file, ZipArchive::CREATE)!==TRUE) {
            $GLOBALS["PROCESS_LOG"]->writeLog("Migration Process - Utilities - Zip - Impossibile aprire il file : " . $file);
        }
        
        ;
        $result = true;
       
        if(!$zip->addFile($fileToAdd))
        {
            $GLOBALS["PROCESS_LOG"]->writeLog("Migration Process - Utilities - Zip - Multipart Zip Failed");
            $result =  false;
        }
        return $result;
    }
    static function zipFiles($packageName, $filesToZip = [], $destinationFolder)
    {

        $zip = new ZipArchive();
        if (!$zip->open($destinationFolder . $packageName . ".zip", ZipArchive::CREATE | ZipArchive::OVERWRITE)) {
            $GLOBALS["PROCESS_LOG"]->writeLog("Migration Process - Utilities - Zip - Can't open Zip Archive");
            return false;
        }

        $hasError = false;
        foreach ($filesToZip as $fileToZip) {
            // se il path non esiste si interrompe
            if (!file_exists($fileToZip["path"])) {
                $GLOBALS["PROCESS_LOG"]->writeLog("Migration Process - Utilities - Zip - File DOES NOT EXIST " . $fileToZip["path"]);
                $hasError = true;
                break;
            }

            if (is_dir($fileToZip["path"]) ) {

                if (!self::addFolderToArchive($zip, $fileToZip, true,$_SESSION['plugin_folder_exclude'])) {
                    return false;
                }
            } else {
                if (!$zip->addFile($fileToZip["path"], $fileToZip["name"])) {
                    $GLOBALS["PROCESS_LOG"]->writeLog("Migration Process - Utilities - Zip - Can't add to Zip Archive [" . $fileToZip["name"] . "] " . $fileToZip["path"]);
                    $hasError = true;
                    break;
                }
            }
        }

        if ($hasError) {
            return false;
        }

        // Zip archive will be created only after closing object
        if (!$zip->close()) {
            $GLOBALS["PROCESS_LOG"]->writeLog("Migration Process - Utilities - Zip - Can't close Zip Archive");
            return false;
        }
        return true;
    }

    static function zipFolder($folderToZip, $destinationFolder, $foldersToExclude)
    {
        $zip = new ZipArchive();
        if (!$zip->open($destinationFolder . $folderToZip["name"] . ".zip", ZipArchive::CREATE | ZipArchive::OVERWRITE)) {
            $GLOBALS["PROCESS_LOG"]->writeLog("Migration Process - Utilities - Zip - Can't open Zip Archive");
            return false;
        }

        if (!self::addFolderToArchive($zip, $folderToZip, false, $foldersToExclude)) {
            return false;
        }

        // Zip archive will be created only after closing object
        if (!$zip->close()) {
            $GLOBALS["PROCESS_LOG"]->writeLog("Migration Process - Utilities - Zip - Can't close Zip Archive");
            return false;
        }
        return true;
    }


    private static function addFolderToArchive($zip, $folderToZip, $addPrefix = false, $foldersToExclude = [])
    {
        $folderName = $addPrefix ? $folderToZip["name"] . DIRECTORY_SEPARATOR : "";
        $rootPath = realpath($folderToZip["path"]);
        // Create recursive directory iterator

        /** @var SplFileInfo[] $files */
        $files = new RecursiveIteratorIterator(
            new RecursiveDirectoryIterator($rootPath),
            RecursiveIteratorIterator::LEAVES_ONLY
        );

        $hasError = false;

        foreach ($files as $name => $file) {
            // Skip directories (they would be added automatically)
            if (!$file->isDir()) {
                // Get real and relative path for current file
                $filePath = $file->getRealPath();
                $filePath = str_replace('\\', '/', $filePath);
                $canUse = true;
                foreach ($foldersToExclude as $val) {
                    $canUse = $canUse && !stristr($filePath, str_replace('\\', '/', $val) );
                }

                if ($canUse) {
                    // Add current file to archive
                    //remove plugin aruba from archive && (strpos($filePath,WP2WP_PLUGIN_DIR_NAME)===false)
                    if((strpos($filePath,PROCESS_FOLDER_NAME)===false) ){

                    $relativePath = substr($folderName . $filePath, strlen($rootPath) + 1);

                    if (!$zip->addFile($filePath,$relativePath)) {
                        $GLOBALS["PROCESS_LOG"]->writeLog("Migration Process - Utilities - Zip - Can't add to Zip Archive [" . $filePath . "] " . $relativePath);
                        $hasError = true;
                        break;
                    }
                       // $zip->setEncryptionName($filePath, ZipArchive::EM_AES_256);


                    //$zip->addFile($filePath, $relativePath);
                    }
                }
            }
        }
        return !$hasError;
    }

    static function encryptArray($array, $key, $iv) {
        $data = serialize($array);
        $encrypted = openssl_encrypt($data, 'AES-256-CBC', $key, 0, $iv);
        return $encrypted;
    }
    static function decryptArray($encrypted, $key, $iv) {
        $decrypted = openssl_decrypt($encrypted, 'AES-256-CBC', $key, 0, $iv);
        $array = unserialize($decrypted);
        return $array;
    }
}
