<?php

use ParagonIE\Sodium\Core\Util;

class AWMT_AdminPostActions
{
    private function init()
    {
        $testx = "";
    }
    /**
     * Gestise i redirect ai vari step della migrazione
     */
    public static function goToStep($step, $getArgs = "")
    {
        if (isset(AWMT_STEPS_URL[$step])) {
            wp_redirect(admin_url(AWMT_STEPS_URL[$step]) . $getArgs);
            exit;
        }
    }

    /**
     * Permette di verificare la validità del token di migrazione
     * Questa chiamata viene fatta dal form allo step 1, e permette di andare allo step 2 (se non ci sono errori)
     */
    public static function insertToken()
    {
        //check_admin_referer( 'verify_token' );
        if ( isset( $_REQUEST['add_token_nonce'] ) && wp_verify_nonce( $_REQUEST['add_token_nonce'], 'add_token' ) ) {
        if (empty(esc_attr($_REQUEST['token-input']))) {
            self::goToStep(1);
        }

		if(wp_verify_nonce( sanitize_text_field( wp_unslash ($_REQUEST['add_token_nonce'])), 'add_token' ) ){
          $params = [
            "token" => sanitize_text_field($_REQUEST['token-input'])
          ];
		}else{
			self::goToStep(1,"&err-gen=42400");
		}
        $res = wp_remote_post(
            AWMT_API_URL . "token/verify/" ,
            array(
                'method' => 'POST',
                'timeout' => 45,
                'headers' => [
                   'Content-Type' => 'application/json',
                    'Accept' => '*/*'
                ],
                'body' => wp_json_encode($params)
            )
        );

        $resp_code = wp_remote_retrieve_response_code($res);
        $resp_body = wp_remote_retrieve_body($res);
        $data = json_decode($resp_body, true);

        if ($resp_code!==200) {
            if (isset($data['errorCode'])) {
                self::goToStep(1, "&err-inv-token=" . $data['errorCode']);
            }elseif ($resp_code==400) {
                self::goToStep(1, "&err-inv-token=42001" );
            }else{
                self::goToStep(1, "&err-gen=42400");
            }
        }else {
                $_SESSION['wp2wp-migration-token'] = $_REQUEST['token-input'];
                $_SESSION['wp2wp-jwt'] = $data["access_token"];
                self::goToStep(2);
        }
        } else {
            self::goToStep(1, "&err-gen=42400");
        }
    }

    /**
     * Vengono collezionate le cartelle ed i file selezionati dall utente allo step 3
     * Questa chiamata viene fatta dal form allo step 3, e permette di andare allo step 4 (se non ci sono errori)
     */
    public static function selectFolders()
    {
        if ( isset( $_REQUEST['_wpnonce'] ) && wp_verify_nonce( $_REQUEST['_wpnonce'], 'check_folders' ) ) {
            $foldersToCopy = $_SESSION["required-folders"];

            unset($_SESSION["required-folders"]);
            unset($_REQUEST["action"]);
            $current_folders=$_REQUEST;
            foreach ($current_folders as $key => $val) {
                if($key!=="_wpnonce" && $key!=="_wp_http_referer"){
                  array_push($foldersToCopy, ["name" => $key, "path" => $val]);
                }
            }

        $_SESSION['wp2wp-folders'] = $foldersToCopy;
        //$GLOBALS["FOLDER_LOG"]->writeLog("-----------------SELECTED FOLDER----------------");
       // foreach($foldersToCopy as $key => $val){
          //  $GLOBALS["FOLDER_LOG"]->writeLog("name: ".$key." path: ".$val['path']);
        //}
        //$GLOBALS["FOLDER_LOG"]->writeLog("-----------------SELECTED FOLDER----------------");
        self::goToStep(4);
        }
    }

    /**
     * Vengono collezionate le scelte dell utente allo step 4, in merito all'impostazione della maintenance e backup 
     * Qui viene avviato il processo che si occupa dei vari stati di migrazione
     * Questa chiamata viene fatta dal form allo step 4, e permette di andare allo step 5 (se non ci sono errori)
     */
    public static function setBackupAndMaintenance()
    {

        require_once ABSPATH . 'wp-admin/includes/class-wp-filesystem-base.php';
        require_once ABSPATH . 'wp-admin/includes/class-wp-filesystem-direct.php';
        // phpcs:ignore WordPress.Security.NonceVerification.Recommended
        $_SESSION['wp2wp-active-backup'] = isset($_REQUEST["activeBackup"]);
        // phpcs:ignore WordPress.Security.NonceVerification.Recommended
        $_SESSION['wp2wp-active-maintenance'] = isset($_REQUEST["activeMaintenance"]);

        global $wpdb;
        $dbConn = [
            "dbname" => $wpdb->dbname,
            "dbuser" => $wpdb->dbuser,
            "dbpassword" => $wpdb->dbpassword,
            "dbhost" => $wpdb->dbhost
        ];

        if (!self::generateMigrationFolder()) {
            self::goToStep(4, "&err-gen-folder=1");
            return;
        } 

        $processParams = [
            "token" => $_SESSION['wp2wp-migration-token'],
            "apiUrl" => AWMT_API_URL,
            "jwt" => esc_attr(sanitize_text_field($_SESSION['wp2wp-jwt'])),
            "maintenance" => esc_attr(sanitize_text_field($_SESSION['wp2wp-active-maintenance'])),
            "backup" => esc_attr(sanitize_text_field($_SESSION['wp2wp-active-backup'])),
            "foldersToCopy" => $_SESSION['wp2wp-folders'],
            "foldersToExclude" => [AWMT_PROCESS_FOLDER_PATH,AWMT_PLUGIN_DIR_PATH], //
            "migrationFolderPath" => AWMT_MIGRATION_FOLDER_PATH,
            "logFilePath" => AWMT_FILE_LOG_PATH,
            "migrationFilePath" => AWMT_FILE_MIGRATION_PATH,
            "dbConn" => $dbConn,
        ];

        if(isset($_SESSION['plugin_folder_exclude'])){
            $processParams["foldersToExclude"]=array_merge(AWMT_sanitize_array($processParams["foldersToExclude"]),AWMT_sanitize_array($_SESSION['plugin_folder_exclude']));
        }

      /*  $encriptData = AWMT_Utilities::encryptArray($processParams,AWMT_ENCRYPT_PWD,md5(AWMT_ENCRYPT_PWD));
        $wp_filesystem = new WP_Filesystem_Direct(null);
        $wp_filesystem->put_contents(AWMT_PROCESS_FOLDER_PATH . DIRECTORY_SEPARATOR . 'sss', $encriptData,0664);
      */
        $_SESSION['wp2wp-process-params'] = $processParams;
        //wp_clear_scheduled_hook( 'migration_update' );
        $GLOBALS["AWMT_PROCESS_LOG"] = new AWMT_MigrationLog($processParams["logFilePath"]);
    }


    public static function startStepProcess($step){
       return  self::startProcess($step);
    }



    /**
     * Interrompe la migrazione interrompendo il processo (lato plugin) e fa redirect allo step 5, che mostrerà la scermata opportuna
     */
    public static function stopMigration()
    {
        //$pid = $GLOBALS["AWMT_MIGRATION_FILE_MANAGER"]->getMigrationProcessID();
        //$res = exec("kill -9 $pid", $out);

        $GLOBALS["AWMT_MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_KILLED_BY_USER", "true");

        // viene svuotata (distrutta e ricreata) la cartella contenente tutti i file di migrazione
        if (!self::generateMigrationFolder()) {
            //TODO: gestire errore?
        }
        // si ritorna allo step 5
        self::goToStep(5);
    }

    /**
     * Fa ripartire la migrazione da capo, dal primo stato (5 step)
     * Potrebbe esser gestito il recupero degli step conclusi, e ripartire da li
     */
    public static function restartMigration()
    {
        //self::startProcess();
        //sleep(1);
        //self::goToStep(5);
        $wpFsWrapper = new AWMT_WPFSWrapper();
        $wpFsWrapper->createFile(AWMT_FILE_MIGRATION_PATH, AWMT_FILE_MIGRATION_DEFAULT_CONTENT, 0775);
        unset($_SESSION['wp2wp-migration-token']);
        unset($_SESSION['wp2wp-process-jwt']);
        unset($_SESSION['wp2wp-folders']);
        unset($_SESSION['wp2wp-active-maintenance']);
        unset($_SESSION['wp2wp-active-backup']);
        unset($_SESSION['wp2wp-process-params']);
        self::goToStep(1);
    }


    /**
     * Pulisce il file .migration e fa ripartire la migrazione da capo, primo step
     */
    public static function newMigration()
    {
        $wpFsWrapper = new AWMT_WPFSWrapper();
        $wpFsWrapper->createFile(AWMT_FILE_MIGRATION_PATH, AWMT_FILE_MIGRATION_DEFAULT_CONTENT, 0775);
        unset($_SESSION['wp2wp-migration-token']);
        unset($_SESSION['wp2wp-process-jwt']);
        unset($_SESSION['wp2wp-folders']);
        unset($_SESSION['wp2wp-active-maintenance']);
        unset($_SESSION['wp2wp-active-backup']);
        unset($_SESSION['wp2wp-process-params']);
        self::goToStep(1);
    }
   /* public function  startCron()
    {
        // server-level cron job
        //avvia la schedulazione
        if (!wp_next_scheduled('migration_update')) {
            wp_schedule_event(time(), 'daily', 'migration_update');
            //wp_schedule_single_event(time()+7400, 'migration_update');
        }

        //Debug
        $next_run = wp_next_scheduled( 'migration_update' );
        $testdeljob =  date( 'Y-m-d H:i:s', $next_run );
        //Debug
        //creo l'endpoint
        $endpoint =  parse_url( get_site_url(), PHP_URL_HOST ) . DIRECTORY_SEPARATOR . AWMT_PLUGIN_DIR_NAME . AWMT_CRON_JOB_ENDPOINT; //   wp_remote_get( 'https://www.example.com/wp-json/my-cron-plugin/v1/cron-job'

        $test = get_rest_url();

        sleep(3);

    }*/
/*
    public static function migration_update()
    {
        if(!isset($processParams))
        {
	        $wpfsd = new WP_Filesystem_Direct( false );
	        $encryptedData = $wpfsd->get_contents ( AWMT_PROCESS_FOLDER_PATH . DIRECTORY_SEPARATOR . 'sss' );//file_get_contents(AWMT_PROCESS_FOLDER_PATH . DIRECTORY_SEPARATOR . 'sss');
            //$encryptedData = file_get_contents(AWMT_PROCESS_FOLDER_PATH . DIRECTORY_SEPARATOR . 'sss');
            $processParams = Utilities::decryptArray($encryptedData,"test",md5("test"));
        }
        $GLOBALS["AWMT_PROCESS_LOG"] = new AWMT_MigrationLog($processParams["logFilePath"]);
        $GLOBALS["AWMT_PROCESS_LOG"]->writeLog('Chiamata cron da orchestrator');

        //TODO proccess params is lost when the session is closed, a method must be created in the API to recover the jwt (params) in this phase (take a look on line 109)

        return self::startProcess();

    }*/


    private static function startProcess($_step=null)
    {
        /*if(!isset($processParams))
        {
            require_once ABSPATH . 'wp-admin/includes/class-wp-filesystem-base.php';
            require_once ABSPATH . 'wp-admin/includes/class-wp-filesystem-direct.php';
            $wp_filesystem = new WP_Filesystem_Direct(null);
            $encryptedData =$wp_filesystem->get_contents(AWMT_PROCESS_FOLDER_PATH . DIRECTORY_SEPARATOR . 'sss');
            $processParams = AWMT_Utilities::decryptArray($encryptedData,AWMT_ENCRYPT_PWD,md5(AWMT_ENCRYPT_PWD));
        }*/


        $processParams = $_SESSION['wp2wp-process-params'];

        $GLOBALS["AWMT_PROCESS_LOG"] = new AWMT_MigrationLog($processParams["logFilePath"]);
        $GLOBALS["AWMT_MIGRATION_FILE_MANAGER"] = new AWMT_MigrationFileManager($processParams["migrationFilePath"]);

        $GLOBALS["AWMT_MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_KILLED_BY_USER", "false");
        $GLOBALS["AWMT_MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_ERROR", "false");
        //$GLOBALS["AWMT_MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_PROCESS_ID", $pid);
        $GLOBALS["AWMT_MIGRATION_FILE_MANAGER"]->updateMigrationKey("WP2WP_MIGRATION_COMPLETED", "false");

        $migrator = new AWMT_ArubaMigrator($processParams);

        if(!is_null($_step)){
                return $migrator->start($_step);
        }else{
            $migrator->start();
        }
        sleep(1);
    }

    private static function generateMigrationFolder()
    {
        $wpFsWrapper = new AWMT_WPFSWrapper();
        $wpFsWrapper->deleteFolder(AWMT_MIGRATION_FOLDER_PATH);
        $res = $wpFsWrapper->createFolder(AWMT_MIGRATION_FOLDER_PATH);
        $wpFsWrapper->createFile(AWMT_MIGRATION_FOLDER_PATH."/.htaccess",
	         'Order deny, allow
deny from all', 0775);
        return !is_wp_error($res);
    }

}