<?php

class AWMT_BackupMaintenanceStep extends AWMT_MigrationStep
{
    function __construct()
    {
        parent::__construct(4, esc_html(__("Copy and upload data",'aruba-wp-migration-tool')) );
    }

    public function canBeDone()
    {

        return !empty($_SESSION['wp2wp-migration-token']) && !empty($_SESSION['wp2wp-jwt']) && $_SESSION['wp2wp-valid-requirements'];
    }

    public function draw()
    {

	    wp_enqueue_script(
		    'backup-maintenance-step-action',
		    WP_PLUGIN_URL .'/aruba-wp-migration-tool/src/migration-plugin/assets/js/backup-maintenance-step.js',
		    array('jquery'),
		    filemtime( WP_PLUGIN_DIR.'/aruba-wp-migration-tool/src/migration-plugin/assets/js/backup-maintenance-step.js' ),
		    array(
			    'strategy'  => 'async',
			    'in_footer'=>true
		    )
	    );

	    $js_param = array(
		    'awmt_ajax_url' => esc_url(admin_url( 'admin-ajax.php' )),
		    'awmt_ajax_url_start_process' => esc_url(admin_url('admin-ajax.php?action=start_step_process')),
		    'awmt_ajax_url_migration_error' => esc_url(admin_url('admin-ajax.php?action=setmigrationerror')),
		    'awmt_ajax_url_preparemigrationdetail'=>esc_url(admin_url('admin-ajax.php?action=preparemigrationdetail')),
		    'awmt_ajax_url_setmigrationexited'=>esc_url(admin_url('admin-ajax.php?action=setmigrationexited')),
            'awmt_ajax_url_setmigrationexited_msg'=>esc_html(__("By exiting the Aruba Migration Tool page, the migration procedure will be interrupted and all progress will be lost. Are you sure you want to proceed?","aruba-wp-migration-tool")),
            'awmt_ajax_url_post' => esc_url(admin_url( 'admin-post.php' )),
		    'awmt_ajax_url_step5'=> esc_url(admin_url(AWMT_STEPS_URL[5])),
	    );
	    wp_add_inline_script( 'backup-maintenance-step-action', 'const awmt_backup_maintenance_param = ' . wp_json_encode( $js_param ), 'before' );


		// $errGenFolder = isset($_GET['err-gen-folder']);
        // phpcs:ignore WordPress.Security.NonceVerification.Recommended
        $errGenFolder = isset($_GET['err-gen-folder']);

        if ($errGenFolder) {
            echo "<p class='wp2wp-text-notification'>" . esc_html(__("Unable to create migration folder" , "aruba-wp-migration-tool")) . "<p>";
        }

        $my_allowed = wp_kses_allowed_html( 'post' );
        // form fields - input
        $my_allowed['form'] = array(
            'method' => true,
            'id'    => true,
            'name'  => true,
            'value' => true,
            'type'  => true,
            'action'=>true,

        );
        $my_allowed['input'] = array(
            'class' => true,
            'id'    => true,
            'name'  => true,
            'value' => true,
            'type'  => true,
            'checked'=>true,
            'disabled'=>true
        );
        $my_allowed['div']=array(
            'class' => true,
            'id'    => true,
            'style'=>true
        );


        echo wp_kses("<div id='substep1'>
            <p> ".esc_html(__('Click on “Continue” to enable us to copy your files to a temporary storage space and perform the checks required for a successful migration.', "aruba-wp-migration-tool")). "</p>
 <p class='wp2wp-text-notification'><b>" . esc_html(__("Depending on the size of folders to be copied and the connection, this procedure may take several minutes." , "aruba-wp-migration-tool")) . "</b></p>
            <form id='migrationpreparation' method='GET' action='" . esc_url(admin_url('admin-post.php')) . "'>
               
                <div>
                    <input type='checkbox' id='activeMaintenance' name='activeMaintenance' checked value='activeMaintenance'>
                    <label for='activeMaintenance' class='wp2wp-text-bold'>" . esc_html(__("I want to place my WordPress site in Maintenance mode" , "aruba-wp-migration-tool")) . "</label>
                    <p class='wp2wp-pl'>" .esc_html( __("This WordPress site will be placed in maintenance mode, making it unavailable for use during the migration, thereby ensuring that data is synchronized correctly and there is no misalignment between the old and the new site. Activating this mode will mean that the site will only become active again once it is published.",'aruba-wp-migration-tool')) . "</p> 
                </div>
                <input type='hidden' name='action' value='backup_maintenance'>
                <hr class='wp2wp-mb wp2wp-mt'>
                <input class='button-primary pull-right' type='submit' type='submit' value='".esc_html(__("Continue","aruba-wp-migration-tool"))."'>
                <a class='button-secondary pull-right wp2wp-mr' href='?page=aruba-wp-migration-tool-migration&step=3'>".esc_html(__("Back","aruba-wp-migration-tool"))."</a>
            </form>
            </div>
            
            <div id='substep2' style='display: none;'>
            <div class='wp2wp-panel' style='padding: 20px'>
                <h2 class='wp2wp-text-center actiontitle'>".esc_html(__("Copying and uploading data","aruba-wp-migration-tool"))."
                <span>.</span><span>.</span><span>.</span>
                </h2>
            
                <div id='LiteralBarraStato'>
                  <div class='status-bar'>
                  
                    <div class='status-line'></div>
                    <div id='0' class='status-dot orange' style='top:0%;'>
                      <div class='status-label current'>".esc_html(__("Maintenance management","aruba-wp-migration-tool"))." </div> </div>
                   
                    <div class='status-line'></div>
                    <div id='1' class='status-dot white' style='top:20%;'>
                      <div class='status-label'>".esc_html(__("Database dump","aruba-wp-migration-tool"))." </div> </div>
                    
                     <div class='status-line'></div>
                     <div id='2' class='status-dot white' style='top:40%;'>
                       <div class='status-label'>".esc_html(__("Package creation","aruba-wp-migration-tool"))."</div> 
                     </div>
            
                    <div class='status-line'></div>
                    <div id='3' class='status-dot white' style='top:60%;'>
                      <div class='status-label'>".esc_html(__("Uploading packages","aruba-wp-migration-tool"))." </div> 
                    </div>
              
                    
                    <div class='status-line'></div>
                    <div id='4' class='status-dot white' style='top:80%;'>
                      <div class='status-label'>".esc_html(__("Check import service","aruba-wp-migration-tool"))." </div> 
                    </div>
                
                    
                    <div class='status-line'></div>
                    <div id='5' class='status-dot white' style='top:100%;'>
                      <div class='status-label'>".esc_html(__("Completion of preliminary operations","aruba-wp-migration-tool"))." </div> 
                    </div>        
                  </div>
                </div>
                 <hr class='wp2wp-mt wp2wp-mb'>
                <p class='wp2wp-text-center'><b>".esc_html(__("You must keep this page open so as not to interrupt the process.","aruba-wp-migration-tool"))."</b></p>
                <p class='wp2wp-text-center'>".esc_html(__("Depending on the size of the folders to be uploaded and your connection, the procedure may take up to a few minutes.","aruba-wp-migration-tool"))."</p>
          </div>
</div>
",$my_allowed);
    }
}