<?php

class AWMT_CheckFoldersStep extends AWMT_MigrationStep
{
    function __construct()
    {
        parent::__construct(3, __("WordPress folders",'aruba-wp-migration-tool') );
    }
    public function canBeDone()
    {
        return !empty($_SESSION['wp2wp-migration-token']) && !empty($_SESSION['wp2wp-jwt']) && $_SESSION['wp2wp-valid-requirements'];
    }

    public function draw()
    {

	    wp_enqueue_script(
		    'check-folder-step',
		    WP_PLUGIN_URL .'/aruba-wp-migration-tool/src/migration-plugin/assets/js/check-folder-step.js',
		    array('jquery'),
		    filemtime( WP_PLUGIN_DIR.'/aruba-wp-migration-tool/src/migration-plugin/assets/js/check-folder-step.js' ),
		    array(
			    'strategy'  => 'async',
			    'in_footer'=>true
		    )
	    );

        $dirContent = $this->getFolderContent();
        $requiredFinded = [];
        $my_allowed = wp_kses_allowed_html( 'post' );
        // form fields - input
        $my_allowed['form'] = array(
            'method' => true,
            'id'    => true,
            'name'  => true,
            'value' => true,
            'type'  => true,
            'action'=>true,

        );
        $my_allowed['input'] = array(
            'class' => true,
            'id'    => true,
            'name'  => true,
            'value' => true,
            'type'  => true,
            'checked'=>true,
            'disabled'=>true
        );

        echo wp_kses("<p>" . esc_html(__("Select the folders you want to transfer to your new WordPress site." , "aruba-wp-migration-tool")) . "</p>
<p>" . esc_html(__("WordPress files must be included in the migration." , "aruba-wp-migration-tool") ). "</p>
<p>" . esc_html(__("Remember that:" , "aruba-wp-migration-tool")) . "</p>
<p><ul style='list-style: square;list-style-position: inside;'>
<li>" . esc_html(__("Cache folders are not used for migration, so we do not recommend selecting them" , "aruba-wp-migration-tool")) . "</li>
<li>" . esc_html(__("only active plugins will be migrated" , "aruba-wp-migration-tool")) . "</li>
</ul></p>
              <form id='foldercheck' method='GET' action='" . esc_url(admin_url('admin-post.php')) . "'>
              <div class='wp2wp-panel' >
                <table class='wp2wp-folders-view'>
                <thead style='text-align: left;  height:30px;'>
                <tr>
                <th style='padding-left:100px;'>".esc_html(__("File name","aruba-wp-migration-tool"))."</th>
                <th>".esc_html(__("Type","aruba-wp-migration-tool"))."</th>
                <th>".esc_html(__("Size","aruba-wp-migration-tool"))." </th>
                </tr>
                </thead>
                
                <tbody >",$my_allowed);

        foreach ($dirContent as $item) {
            $liClass = $item["isDir"] ? "is-dir" : "is-file";
            $valid = array_search($item["path"], AWMT_REQUIRED_FOLDERS_FILES) === false;

            if (!$valid) {
                $r = [
                    "name" => $item["path"],
                    "path" => $item["abspath"]
                ];
                array_push($requiredFinded, $r);
                echo wp_kses("<tr class='".esc_attr($liClass)." disabled' >
                       <td >
                        <input type='checkbox' checked='true' disabled>
                        <label>" . esc_html($item["path"]) . "</label>
                    </td>
                    <td>".(is_dir($item["abspath"])?'Directory':'File'." ".esc_html(pathinfo($item["abspath"], PATHINFO_EXTENSION)) )."</td>
                     <td>" .(is_dir($item["abspath"])?esc_html($this->format_size($this->folderSize($item["abspath"]))):esc_html($this->format_size(filesize($item["abspath"]))) )."</td>
                    </tr>",$my_allowed);
            } else {
                echo wp_kses("<tr class='".esc_attr($liClass)."'>
                      <td >
                            <input type='checkbox' checked='true' id='" . esc_attr($item["path"]) . "' name='" . esc_attr( $item["path"]) . "' value='" . esc_attr( $item["abspath"] ). "'>
                            <label for='" .  esc_attr($item["path"]) . "'>" .  esc_attr($item["path"]) . "</label>
                        </td>
                        <td>".(is_dir($item["abspath"])?'Directory':'File'." ".esc_html(pathinfo($item["abspath"], PATHINFO_EXTENSION) ))."</td>
                    <td>" .(is_dir($item["abspath"])?esc_html($this->format_size($this->folderSize($item["abspath"]))):esc_html($this->format_size(filesize($item["abspath"]))) )."</td>
                        </tr>",$my_allowed);
            }
        }
        echo wp_kses("</tbody></table>
                </div>
                <p style='font-weight: bold;'>".esc_html(__("Depending on the size of the folders to be uploaded and your connection, the procedure may take up to a few minutes.","aruba-wp-migration-tool"))."</p>
                
                <input type='hidden' name='action' value='select_folders'>
                <hr class='wp2wp-mb wp2wp-mt'>
                 <button class='button-primary pull-right' type='button' id='submitbutton'><label>".esc_html(__('Continue','aruba-wp-migration-tool'))."</label>&nbsp;<i class='fa fa-circle-o-notch fa-spin' style='display:none'></i></button>
                <a class='button-secondary pull-right wp2wp-mr' href='?page=aruba-wp-migration-tool-migration&step=2'>".esc_html(__("Back","aruba-wp-migration-tool"))."</a>
            ".wp_nonce_field( 'check_folders','_wpnonce',true,false)."
            </form>",$my_allowed);


        $_SESSION["required-folders"] = AWMT_sanitize_array($requiredFinded);
    }
 private function folderSize ($dir)
{
    $size = 0;
    foreach (glob(rtrim($dir, '/').'/*', GLOB_NOSORT) as $each) {
        $size += is_file($each) ? filesize($each) : $this->folderSize($each);
    }

    return $size;
}

private function format_size($size) {
    $mod = 1024;
    $units = explode(' ','B KB MB GB TB PB');
    for ($i = 0; $size > $mod; $i++) {
        $size /= $mod;
    }

    return round($size, 2) . ' ' . $units[$i];
}
    private function getFolderContent()
    {
        $dirContent = array_diff(scandir(ABSPATH), array('.', '..'));
        // per rimuovere le cartele e i file da escludere 
        $dirContent = array_filter($dirContent, function ($var) {
            return  $var !== "." && $var !== ".." && array_search($var, AWMT_EXCLUDED_FOLDERS_FILES) === false;
        });

        $dirContent = array_map(function ($v) {
            return ["isDir" => is_dir(ABSPATH . $v), "path" => $v, "abspath" => ABSPATH . $v];
        }, $dirContent);

        // per ordinare prima le cartelle e dopo i file
        usort($dirContent, function ($a, $b) {
            return $b['isDir'] - $a['isDir'];
        });

        return $dirContent;
    }
}
